% Forced Duffing oscillator :
%
%   Second order equation : x'' + epsilon x' + x + x^3 = alpha*cos(lambda*t)
%   First order recast to compute the stability of the solution :
%       x' = y 
%       y' = - epsilon*y - x - x^3 - alpha*cos(lambda t)  (nz=2)
%
%   The first order recast is written in the formalism f(X) - X' = Forcing(t) to
%   have a correct stability analysis of the problem :
%
%    y                   - x'  = 0
%  - x - epsilon y - r*x - y'  = - alpha cos(lambda t)
%    r             - x^2       = 0
%
%   The last variable r = x^2 allows to write the system in the quadratic
%   form needded.
%
%    z(t) = [ x(t) ; y(t) ; r(t) ]  (nz_tot=3)  % unknown of the DAE
%    x = [ x_0 ;  x_C1 ;  x_C2 ; ...  ; x_CH ; x_S1 ;  x_S2 ; ...  ; x_SH ]  Fourier coef (colum) vector for x
%    Z= [ x , y , r ]  matrix of unknown Fourier coefficients
%    Z= reshape(Z, nz_tot*(2*H+1),1)  -> column vector of unknown Fourier coefficients

global U Section Diagram    % Global variables to export point from the diagram. 

addpath('../../SRC')

%% Parameters of the system
nz= 2;     % number of main equations of the DAE system
nz_aux =1;  % number of auxiliary equations of the DAE system
H = 30;     % number of harmonics

% specific parameters
parameters.alpha = 1;
parameters.epsilon = .05;

%% initialization of the system
sys=SystHBQ(nz,nz_aux,H,@equations,@point_display,@global_display,parameters,'forced');

%% starting point
omega=0.21;lambda=omega;

Z0  =randn(2*H+1,sys.nz_tot)*1e-7;
Z0(2,1) = 0.7;
Z0(:,2) = omega*sys.D(Z0(:,1));      % y = x' in the Fourier domain.
Z0(:,3) = sys.Prod(Z0(:,1),Z0(:,1)); % r = x*x in the Fourier domain.

U0tot = sys.init_U0(Z0,omega,lambda);

% The bifurcation diagram represents
% first cosine of x with respect to lambda and
% first sine of x with respect to lambda :
dispvars = [sys.neq+1 2; ...
    sys.neq+1 2+H];

%% Launch of Manlab with options
Manlab('sys'       ,sys , ...
    'U0value'         ,U0tot, ...
    'ANMthreshold'    ,1e-10 , ...
    'NRthreshold'     ,1e-7, ...
    'StabilityCheck'  ,1, ...
    'displayvariables',dispvars);     % MANLAB run


%% Or compute the full diagram (90 continuation steps) without graphical interface and save it in "Diagram" structure
% Diagram = Manlab_script('sys'       ,sys , ...
%     'nb_step'         ,90, ...
%     'U0value'         ,sys.init_U0(Z0,omega,lambda) , ...
%     'ANMthreshold'    ,1e-10 , ...
%     'NRthreshold'     ,1e-7, ...
%     'StabilityCheck'  ,1, ...
%     'displayvariables',dispvars);     % MANLAB run

% 
% %%% Post-processing :
%  
% % Global display of all the diagram using user defined global_display function.
% for i=1:length(Diagram)
%     global_display(sys,Diagram{i});
% end
% 
% % Display of the last point of the diagram using user defined point_display function.
% point_display(sys,Diagram{end}.Uend);

