% Forced Van der Pol oscillator.
%
% From Journal of Sound and Vibration, vol 394, year 2017, pages 434-450, 
% Continuation of quasi-periodic solutions with two-frequency Harmonic
% Balance Method, authors : L.Guillot; P.Vigu; C.Vergez; B.Cochelin
%
%    First order model of a forced van der pol oscillator :
%   x' = y 
%   y' + (- mu1 + mu2 x + x^2) y + x = F cos(lambda t) )  (nz=2)
%
% The derivated terms should be with a -1 sign and the forced term is
% isolated on the right hand side, as follows :
%
%    y                    - x'  = 0
%  - x              + ry  - y'  = - F cos(lambda t)
%  mu1 - r - mu2*x  - x^2       = 0
%
%    z(t) = [ x(t) ; y(t) ; r(t) ]  (nz_tot=3)  % unknown of the DAE
%    x = [ x_0 ;  x_C1 ;  x_C2 ; ...  ; x_CH ; x_S1 ;  x_S2 ; ...  ; x_SH ]  Fourier coef (colum) vector for x
%    Z= [ x , y , r ]  matrix of unknown Fourier coefficients
%    Z= reshape(Z, nz_tot*(2*H+1),1)  -> column vector of unknown Fourier coefficients

global U Section Diagram    % Global variables to export point from the diagram. 

addpath('../../SRC')

%% Parameters of the system
nz= 2;     % number of main equations of the DAE system
nz_aux =1;  % number of auxiliary equations of the DAE system
H = 15;     % number of harmonics

% specific parameters
parameters.mu1 = 0.1;
parameters.mu2 = 0.1;
parameters.F   = 1;

%% initialization of the system
sys=SystHBQ(nz,nz_aux,H,@equations,@point_display,@global_display,parameters,'forced');

%% starting point
omega = 0.03; lambda=omega;

Z0  =randn(2*H+1,sys.nz_tot)*1e-7;
Z0(2,1) = 0.5;
%%% Auxiliary variables :
Z0(:,2) = omega*sys.D(Z0(:,1));
Z0(1,3) = parameters.mu1;
Z0(:,3) = Z0(:,3) - parameters.mu2*Z0(:,1) - sys.Prod(Z0(:,1),Z0(:,1));

U0tot = sys.init_U0(Z0,omega,lambda);

% The bifurcation diagram represents
% first cosine of x with respect to lambda and
% first sine of x with respect to lambda :
dispvars = [sys.neq+1 2; ...
    sys.neq+1 2+H];

%% Launch of Manlab with options
Manlab('sys'       ,sys , ...
    'U0value'         ,U0tot, ...
    'StabilityCheck'  ,1, ...
    'displayvariables',dispvars);     % MANLAB run

